@@ -1,12 +1,21 @@
 # coding: utf-8
 
-from urllib import quote
-from url import URLAccumulator
-from xml.dom import minidom
+### WIKIPEDIA ########################################################################################
+# Code for querying Wikipedia and parsing articles.
+# The parser is as-is, it handles a lot but not everything.
+
+# Author: Tom De Smedt.
+# Copyright (c) 2007 by Tom De Smedt.
+# See LICENSE.txt for details.
+
 import re
+from xml.dom import minidom
+from urllib import quote
+
+from url import URLAccumulator
 from html import replace_entities, strip_tags
-import mimetex
 from cache import Cache
+import mimetex
 
 def clear_cache():
     Cache("wikipedia").clear()
@@ -301,6 +310,7 @@ languages = {
 }
 
 ### WIKIPEDIALINK ####################################################################################
+# Currently not in use.
 
 class WikipediaLink:
     
@@ -311,7 +321,9 @@ class WikipediaLink:
         self.display = display
         
     def __str__(self):
-        
+        return self.page.encode("utf-8")
+    
+    def __unicode__(self):
         return self.page
         
 ### WIKIPEDIAPARAGRAPH ###############################################################################
@@ -330,7 +342,10 @@ class WikipediaParagraph(list):
         self.children = []
         
     def __str__(self):
-        
+        s = "\n\n".join(self)
+        return s.encode("utf-8")
+
+    def __unicode__(self):
         s = "\n\n".join(self)
         return s
  
@@ -346,7 +361,9 @@ class WikipediaImage:
         self.properties = properties
         
     def __str__(self):
-        
+        return self.path.encode("utf-8")
+
+    def __unicode__(self):
         return self.path
 
 ### WIKIPEDIAREFERENCES ##############################################################################
@@ -376,8 +393,11 @@ class WikipediaReference:
             if value != "":
                 s += value.rstrip(".,") + ", "
         
-        s = s.strip(", \n").encode("utf-8")
-        return s
+        s = s.strip(", \n")
+        return s.encode("utf-8")
+        
+    def __unicode__(self):
+        return str(self).decode("utf-8")
 
 ### WIKIPEDIATABLE ###################################################################################
 
@@ -406,16 +426,19 @@ class WikipediaTableCell(unicode):
 
 class WikipediaPage:
     
-    def __init__(self, title, markup, light=False):
+    def __init__(self, title, markup, light=False, full_strip=True):
         
         """ Wikipedia page parser.
         
         The expected markup is the stuff in Wikipedia's edit textarea.
+        With light=True, it will onlt parse links to other articles (which is faster).
+        With full_strip=False, it will preserve some HTML markup (links, bold, italic).
         
         """
         
         self.title = title
         self.markup = markup
+        self.full_strip = full_strip
         
         self.disambiguation = []
         self.categories = []
@@ -464,9 +487,14 @@ class WikipediaPage:
         return str
         
     def __str__(self):
-        
-        s = "\n\n".join([str(p) for p in self.paragraphs])
+        s = ""
+        for p in self.paragraphs:
+            s += (p.title.encode("utf-8") + "\n\n").lstrip("\n")
+            s += (str(p) + "\n\n").lstrip("\n")
         return s
+    
+    def __unicode__(self):
+        return str(self).decode("utf-8")
 
     def parse(self, light=False):
 
@@ -517,12 +545,18 @@ class WikipediaPage:
         stripping images and references and the like.
         Does some commonsense maintenance as well,
         like collapsing multiple spaces.
+        If you specified full_strip=False for WikipediaPage instance,
+        some markup is preserved as HTML (links, bold, italic).
         
         """
         
         # Strip bold and italic.
-        markup = markup.replace("'''", "")
-        markup = markup.replace("''", "")
+        if self.full_strip:
+            markup = markup.replace("'''", "")
+            markup = markup.replace("''", "")
+        else:
+            markup = re.sub("'''([^']*?)'''", "<b>\\1</b>", markup)
+            markup = re.sub("''([^']*?)''", "<i>\\1</i>", markup)
         
         # Strip image gallery sections.
         markup = re.sub(self.re["gallery"], "", markup)
@@ -534,7 +568,11 @@ class WikipediaPage:
         
         # Strip links, keeping the display alias.
         # We'll strip the ending ]] later.
-        markup = re.sub(r"\[\[[^\]]*?\|", "", markup)                 
+        if self.full_strip:
+            markup = re.sub(r"\[\[[^\]]*?\|", "", markup)
+        else:
+            markup = re.sub(r"\[\[([^]|]*|)\]\]", '<a href="\\1">\\1</a>', markup)
+            markup = re.sub(r"\[\[([^]|]*|)\|([^]]*)\]\]", '<a href="\\1">\\2</a>', markup)    
 
         # Strip translations, users, etc.
         markup = re.sub(self.re["translation"], "", markup)
@@ -568,7 +606,7 @@ class WikipediaPage:
         markup = re.sub("</{0,1}ref.*?>", "", markup)                      # b
         markup = re.sub("<ref name=\".*?\" {0,1}/>", "", markup)           # c
         markup = re.sub(self.ref+"\(([0-9]*?)\)", "[\\1] ", markup)        # d
-        markup = re.sub("\] ([,.\"\?])", "]\\1", markup)                   # e
+        markup = re.sub("\] ([,.\"\?\)])", "]\\1", markup)                 # e
         markup = re.sub(self.re["comment"], "", markup)                    # f
         markup = re.sub("\{\{latinx\|(.*?)\}\}", "\\1", markup)            # g
         markup = re.sub("\{\{ME-ref.*?\}\}", "", markup)                   # h
@@ -587,7 +625,8 @@ class WikipediaPage:
         markup = markup.replace(" .", ".")
         
         # Strip all HTML except <math> tags.
-        markup = strip_tags(markup, exclude=["math"], linebreaks=True)
+        if self.full_strip:
+            markup = strip_tags(markup, exclude=["math"], linebreaks=True)
         
         markup = markup.strip()
         return markup
@@ -1507,7 +1546,7 @@ class WikipediaSearch(WikipediaPage, URLAccumulator):
         return url
     
     def __init__(self, q, language="en", light=False, wait=10, asynchronous=False, cached=True,
-                 case_sensitive=False):
+                 case_sensitive=False, full_strip=True):
         
         """ A download manager for Wikipedia pages.
         
@@ -1521,6 +1560,7 @@ class WikipediaSearch(WikipediaPage, URLAccumulator):
         """
         
         self._light = light
+        self._full_strip = full_strip
         
         if cached: 
             cache = "wikipedia"
@@ -1542,16 +1582,15 @@ class WikipediaSearch(WikipediaPage, URLAccumulator):
             rev = dom.getElementsByTagName("rev")[0]
             data = rev.childNodes[0].nodeValue.strip()
         except:
-            print data
             if not self.error:
                 self.error = WikipediaPageMissing()
             data = ""
 
-        WikipediaPage.__init__(self,  title, data, light=self._light)
+        WikipediaPage.__init__(self,  title, data, light=self._light, full_strip=self._full_strip)
 
 def search(q, language="en", light=False, wait=10, asynchronous=False, cached=True, 
-           case_sensitive=False):
-    return WikipediaSearch(q, language, light, wait, asynchronous, cached, case_sensitive)
+           case_sensitive=False, full_strip=True):
+    return WikipediaSearch(q, language, light, wait, asynchronous, cached, case_sensitive, full_strip)
 
 ######################################################################################################
 # Some interesting things...
